@@ -1,16 +1,21 @@
 import base64
-from dataclasses import dataclass, field
-from datetime import datetime, timedelta, timezone
+import dataclasses
+import datetime
 from dateutil.parser import isoparse
 import hashlib
+import typing
 
-ZeroTime = datetime(1, 1, 1, 0, 0, tzinfo=timezone.utc)
+# ZeroTime is the zero time used by default for spin types.
+ZeroTime = datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc)
 
+
+# SHA256 creates a sum of the data
 def SHA256(b: bytes):
     m = hashlib.sha256()
     m.update(b)
     return m.hexdigest()
 
+
 """ CitizenName is a helper type for a spin CitizenName """
 CitizenName = str
 """ KeyType is a helper type for a spin KeyType """
@@ -19,7 +24,7 @@ KeyType = str
 KeyName = str
 
 
-@dataclass
+@dataclasses.dataclass
 class Key:
     """
     Key is a helper class for a spin Key.
@@ -30,9 +35,10 @@ class Key:
     name: KeyName = ""
     data: str = ""
     meta: str = ""
-    created_at: datetime = ZeroTime
-    expires_at: datetime = ZeroTime
+    created_at: datetime.datetime = ZeroTime
+    expires_at: datetime.datetime = ZeroTime
 
+    @staticmethod
     def from_json(j: dict):
         k = Key()
         k.unmarshal_json(j)
@@ -69,11 +75,12 @@ class Key:
         j["ExpiresAt"] = self.expires_at.isoformat()
 
 
-@dataclass
+@dataclasses.dataclass
 class PrivateKey:
-    key: Key = Key()
+    key: Key = dataclasses.field(default_factory=Key)
     private: str = ""
 
+    @staticmethod
     def from_json(j: dict):
         k = PrivateKey()
         k.unmarshal_json(j)
@@ -95,11 +102,12 @@ class PrivateKey:
         return j
 
 
-@dataclass
+@dataclasses.dataclass
 class KeyWhichRequest:
     public: str = ""
     private: str = ""
 
+    @staticmethod
     def from_json(j: dict):
         r = KeyWhichRequest()
         r.unmarshal_json(j)
@@ -122,11 +130,12 @@ class KeyWhichRequest:
         return j
 
 
-@dataclass
+@dataclasses.dataclass
 class KeyWhichResponse:
-    key: Key = None
+    key: typing.Optional[Key] = None
     error: str = ""
 
+    @staticmethod
     def from_json(j: dict):
         r = KeyWhichResponse()
         r.unmarshal_json(j)
@@ -149,12 +158,13 @@ class KeyWhichResponse:
         return j
 
 
-@dataclass
+@dataclasses.dataclass
 class KeyTempRequest:
     public: str = ""
     private: str = ""
     duration: int = 0  # nanoseconds
 
+    @staticmethod
     def from_json(j: dict):
         r = KeyTempRequest()
         r.unmarshal_json(j)
@@ -180,12 +190,13 @@ class KeyTempRequest:
         return j
 
 
-@dataclass
+@dataclasses.dataclass
 class KeyTempResponse:
-    key: Key = None
+    key: typing.Optional[Key] = None
     private: str = ""
     error: str = ""
 
+    @staticmethod
     def from_json(j: dict):
         r = KeyTempResponse()
         r.unmarshal_json(j)
@@ -225,12 +236,13 @@ SeqBase = 1
 MaxBlockSize = 10 * (1024 * 1024)
 
 
-@dataclass
+@dataclasses.dataclass
 class DirBlock:
     ref: Ref = ""
     offset: int = 0
     size: int = 0
 
+    @staticmethod
     def from_json(j: dict):
         r = DirBlock()
         r.unmarshal_json(j)
@@ -256,15 +268,16 @@ class DirBlock:
         return j
 
 
-@dataclass
+@dataclasses.dataclass
 class DirEntry:
     type: DirEntryType = EntryMissing
-    blocks: list[DirBlock] = field(default_factory=list)
+    blocks: list[DirBlock] = dataclasses.field(default_factory=list)
     citizen: str = ""
     path: Path = ""
-    time: datetime = ZeroTime
+    time: datetime.datetime = ZeroTime
     sequence: int = 0
 
+    @staticmethod
     def from_json(j: dict):
         r = DirEntry()
         r.unmarshal_json(j)
@@ -298,19 +311,23 @@ class DirEntry:
         j["Sequence"] = self.sequence
         return j
 
+    def size(self):
+        return sum([b.size for b in self.blocks])
+
 
 DirOperation = str
 
-MissingDirOperation = ""
-PutDirOperation = "put"
-DelDirOperation = "del"
+MissingDirOperation: DirOperation = ""
+PutDirOperation: DirOperation = "put"
+DelDirOperation: DirOperation = "del"
 
 
-@dataclass
+@dataclasses.dataclass
 class DirOp:
-    operation: DirOperation = DirOperation()
-    dir_entry: DirEntry = DirEntry()  # embedded
+    operation: DirOperation = MissingDirOperation
+    dir_entry: DirEntry = dataclasses.field(default_factory=DirEntry)  # embedded
 
+    @staticmethod
     def from_json(j: dict):
         r = DirOp()
         r.unmarshal_json(j)
@@ -343,12 +360,13 @@ GetBitOperation = "get"
 PutBitOperation = "put"
 
 
-@dataclass
+@dataclasses.dataclass
 class BitOp:
-    operation: BitOperation = BitOperation()
+    operation: BitOperation = MissingBitOperation
     ref: Ref = ""
-    bytes: bytes = None
+    bytes: bytes = b""
 
+    @staticmethod
     def from_json(j: dict):
         r = BitOp()
         r.unmarshal_json(j)
@@ -359,7 +377,7 @@ class BitOp:
             self.type = BitOperation(j["Operation"])
         if "Ref" in j:
             self.ref = Ref(j["Ref"])
-        if "Bytes" in j:
+        if "Bytes" in j and j["Bytes"] is not None:
             self.bytes = base64.b64decode(j["Bytes"])
 
     def to_json(self):
@@ -374,12 +392,13 @@ class BitOp:
         return j
 
 
-@dataclass
+@dataclasses.dataclass
 class RefData:
     ref: Ref = ""
     volatile: bool = False
     duration: int = 0
 
+    @staticmethod
     def from_json(j: dict):
         r = BitOp()
         r.unmarshal_json(j)
@@ -405,12 +424,13 @@ class RefData:
         return j
 
 
-@dataclass
+@dataclasses.dataclass
 class BitApplyRequest:
     public: str = ""
     private: str = ""
-    ops: list[BitOp] = field(default_factory=list)
-    
+    ops: list[BitOp] = dataclasses.field(default_factory=list)
+
+    @staticmethod
     def from_json(j: dict):
         r = BitApplyRequest()
         r.unmarshal_json(j)
@@ -436,18 +456,45 @@ class BitApplyRequest:
         return j
 
 
-@dataclass
+@dataclasses.dataclass
 class BitApplyOutcome:
-    ref_data: RefData = RefData()
-    bytes: bytearray = None
+    ref_data: RefData = dataclasses.field(default_factory=RefData)
+    bytes: bytes = b""
     Error: str = ""
 
+    @staticmethod
+    def from_json(j: dict):
+        r = BitApplyOutcome()
+        r.unmarshal_json(j)
+        return r
 
-@dataclass
+    def unmarshal_json(self, j: dict):
+        if "RefData" in j:
+            self.ref_data = RefData.from_json(j["RefData"])
+        if "Bytes" in j and j["Bytes"] is not None:
+            self.bytes = base64.b64decode(j["Bytes"])
+        if "Error" in j:
+            self.error = j["Error"]
+
+    def to_json(self):
+        j = {}
+        self.marshal_json(j)
+        return j
+
+    def marshal_json(self, j):
+        self.ref_data.unmarshal_json(j)
+        j["RefData"] = self.ref_data.to_json()
+        j["Bytes"] = base64.b64encode(self.bytes).decode()
+        j["Error"] = self.error
+        return j
+
+
+@dataclasses.dataclass
 class BitApplyResponse:
-    outcomes: list[BitApplyOutcome] = field(default_factory=list)
+    outcomes: list[BitApplyOutcome] = dataclasses.field(default_factory=list)
     error: str = ""
 
+    @staticmethod
     def from_json(j: dict):
         r = BitApplyResponse()
         r.unmarshal_json(j)
@@ -455,7 +502,7 @@ class BitApplyResponse:
 
     def unmarshal_json(self, j: dict):
         if "Outcomes" in j and j["Outcomes"] is not None:
-            self.entries = [BitApplyOutcome.from_json(e) for e in j["Outcomes"]]
+            self.outcomes = [BitApplyOutcome.from_json(o) for o in j["Outcomes"]]
         if "Error" in j:
             self.error = j["Error"]
 
@@ -470,12 +517,13 @@ class BitApplyResponse:
         return j
 
 
-@dataclass
+@dataclasses.dataclass
 class DirApplyRequest:
     public: str = ""
     private: str = ""
-    ops: list[DirOp] = field(default_factory=list)
+    ops: list[DirOp] = dataclasses.field(default_factory=list)
 
+    @staticmethod
     def from_json(j: dict):
         r = DirApplyRequest()
         r.unmarshal_json(j)
@@ -501,11 +549,12 @@ class DirApplyRequest:
         return j
 
 
-@dataclass
+@dataclasses.dataclass
 class DirApplyResponse:
-    entries: list[DirEntry] = field(default_factory=list)
+    entries: list[DirEntry] = dataclasses.field(default_factory=list)
     error: str = ""
 
+    @staticmethod
     def from_json(j: dict):
         r = DirApplyResponse()
         r.unmarshal_json(j)
@@ -528,13 +577,14 @@ class DirApplyResponse:
         return j
 
 
-@dataclass
+@dataclasses.dataclass
 class DirLookupRequest:
     public: str = ""
     private: str = ""
     citizen: CitizenName = ""
     path: Path = ""
 
+    @staticmethod
     def from_json(j: dict):
         r = DirLookupRequest()
         r.unmarshal_json(j)
@@ -563,11 +613,12 @@ class DirLookupRequest:
         return j
 
 
-@dataclass
+@dataclasses.dataclass
 class DirLookupResponse:
-    entry: DirEntry = None
+    entry: typing.Optional[DirEntry] = None
     error: str = ""
 
+    @staticmethod
     def from_json(j: dict):
         r = DirLookupResponse()
         r.unmarshal_json(j)
@@ -590,7 +641,7 @@ class DirLookupResponse:
         return j
 
 
-@dataclass
+@dataclasses.dataclass
 class DirTreeRequest:
     public: str = ""
     private: str = ""
@@ -598,6 +649,7 @@ class DirTreeRequest:
     path: Path = ""
     level: int = 0
 
+    @staticmethod
     def from_json(j: dict):
         r = DirTreeRequest()
         r.unmarshal_json(j)
@@ -629,11 +681,12 @@ class DirTreeRequest:
         return j
 
 
-@dataclass
+@dataclasses.dataclass
 class DirTreeResponse:
-    entries: list[DirEntry] = field(default_factory=list)
+    entries: list[DirEntry] = dataclasses.field(default_factory=list)
     error: str = ""
 
+    @staticmethod
     def from_json(j: dict):
         r = DirTreeResponse()
         r.unmarshal_json(j)
