@@ -3,6 +3,8 @@ from spinsrv import spin
 import json
 import hashlib
 
+print("THIS IS SMOKE TEST 1: IT TESTS THE CLIENTS IN src/spin/spinpy.py")
+
 kc = sp.KeyServerHTTPClient()
 resp = kc.which(
     spin.KeyWhichRequest(
@@ -55,13 +57,17 @@ print(resp)
 
 bc = sp.BitServerHTTPClient()
 data = "Asdf".encode()
-resp = bc.apply(spin.BitApplyRequest(
-    public="proteins",
-    private="proteins",
-    ops=[spin.BitOp(
-        operation=spin.PutBitOperation,
-        ref=spin.SHA256(data),
-        bytes=data,
-    )]
-))
-print(resp)
\ No newline at end of file
+resp = bc.apply(
+    spin.BitApplyRequest(
+        public="proteins",
+        private="proteins",
+        ops=[
+            spin.BitOp(
+                operation=spin.PutBitOperation,
+                ref=spin.SHA256(data),
+                bytes=data,
+            )
+        ],
+    )
+)
+print(resp)
